classdef MuscleI < redmax.Muscle
	%MuscleI A straight line muscle between two bodies.
	%

	properties
	end

	%%
	methods
		%%
		function this = MuscleI(as,ms,bO,xO,bI,xI)
			this = this@redmax.Muscle(as,ms,bO,xO,bI,xI);
			this.useJdotFD = false;
		end

		%%
		function l = computeLength(this)
			dx = this.xIw - this.xOw;
			l = norm(dx);
		end
	end
		
	%%
	methods (Access = protected)
		%%
		function update_(this)
			% Update Mass points
			n = size(this.xs,2);
			for i = 1 : n
				a = this.as(i);
				this.xs(:,i) = (1-a)*this.xOw + a*this.xIw;
			end
		end

		%%
		function [Jam,dJam] = computeJacobian_(this,Jam,dJam,e)
			% va = Jam*vm
			% At each point:
			%      xa = (1-a)*xOw + s*xIw;
			%      va = (1-a)*vOw + s*vIw;
			%         = (1-a)*RO*GO*phiO + s*RI*GI*phiI
			%     Jam = [(1-a)*RO*GO, a*RI*GI]
			% The time derivative is (Section 2.8 in se3.pdf):
			%    dJam = [(1-a)*RO*[wO]*GO, a*RI*[wI]*GI]
			idxMO = this.bO.idxM;
			idxMI = this.bI.idxM;
			RO = this.bO.E_wi(1:3,1:3);
			RI = this.bI.E_wi(1:3,1:3);
			GO = se3.Gamma(this.xO);
			GI = se3.Gamma(this.xI);
			RGO = RO*GO;
			RGI = RI*GI;
			n = size(this.xs,2);
			for i = 1 : n
				a = this.as(i);
				idxAi = this.idxA(3*(i-1)+(1:3));
				Jam(idxAi,idxMO) = (1-a)*RGO;
				Jam(idxAi,idxMI) =     a*RGI;
			end
			if nargout == 2
				if this.useJdotFD
					% For debugging the finite differencing code in the driver
					idxA = this.idxA;
					idxM = [idxMO,idxMI]; % assume the bodies are distinct
					if e == 0
						% This is the first call. Copy Jam into dJam.
						dJam(idxA,idxM) = Jam(idxA,idxM);
					else
						% This is the second call. Apply finite difference, using the value
						% stored in dJam during the first call.
						Jam_ = Jam(idxA,idxM);
						Jam(idxA,idxM) = dJam(idxA,idxM);
						dJam(idxA,idxM) = (Jam_ - Jam(idxA,idxM))/e;
					end
				else
					wO = se3.brac(this.bO.phi(1:3));
					wI = se3.brac(this.bI.phi(1:3));
					RwGO = RO*wO*GO;
					RwGI = RI*wI*GI;
					for i = 1 : n
						a = this.as(i);
						idxAi = this.idxA(3*(i-1)+(1:3));
						dJam(idxAi,idxMO) = (1-a)*RwGO;
						dJam(idxAi,idxMI) =     a*RwGI;
					end
				end
			end
		end

		%%
		function points = getForcePointsHMM(this)
			xO = this.xOw;
			xI = this.xIw;
			dx = xI - xO;
			dx = dx/norm(dx);
			points(1).xw = xO;
			points(1).dw = dx;
			points(1).body = this.bO;
			points(2).xw = xI;
			points(2).dw = -dx;
			points(2).body = this.bI;
		end
		
		%%
		function draw_(this)
			x = [this.xOw(1:3),this.xIw(1:3)];
			c = this.getColor();
			plot3(x(1,:),x(2,:),x(3,:),'-','Color',c,'LineWidth',2);
		end
	end
end
