classdef MuscleNN < redmax.Muscle
	%MuscleNN A superclass for all NN-based muscles
	%

	properties
		jointCount % Number of joints between the two bodies
		NN % Neural network
		xsNN % Positions computed by NN
		JsNN % Jacobians computed by NN
		xsNN_ % Positions for finite differencing (used by III.r; deprecated)
		JsNN_ % Jacobians for finite differencing (used by III.r; deprecated)
		dxsNN % Position derivatives (used by III.m)
		dJsNN % Jacobian derivatives (used by III.m)
	end

	%%
	methods
		%%
		function this = MuscleNN(as,ms,bO,xO,bI,xI,NNfile)
			this = this@redmax.Muscle(as,ms,bO,xO,bI,xI);

			% Count the number of joints from insertion to origin
			this.jointCount = 0;
			joint = bI.joint;
			while joint ~= bO.joint
				this.jointCount = this.jointCount + 1;
				joint = joint.parent;
			end

			if nargin >= 7
				this.NN = load(NNfile);
			end

			n = length(ms);
			this.xsNN = zeros(3,n);
			this.JsNN = zeros(3*this.jointCount,n);
			this.xsNN_ = this.xsNN;
			this.JsNN_ = this.JsNN;
			this.dxsNN = this.xsNN;
			this.dJsNN = this.JsNN;
		end

		%%
		function storeNN(this)
			this.xsNN_ = this.xsNN;
			this.JsNN_ = this.JsNN;
		end

		%%
		function restoreNN(this)
			this.xsNN = this.xsNN_;
			this.JsNN = this.JsNN_;
		end
	end
end
