classdef (Abstract) Wrap < handle
	%Wrap A superclass for wrapping objects
	%   

	properties
		body     % The body this wrapping object is attached to
		E        % Where the wrapping object is wrt the body
		axisSize % Size of display axis
		color    % Display color
	end

	methods
		function this = Wrap(body,E)
			this.body = body;
			this.E = E;
			this.axisSize = 1;
			this.color = [0 1 1];
		end

		function draw(this) %#ok<MANU> 
		end
		
		%%
		function setAxisSize(this,s)
			this.axisSize = s;
		end
		
		%%
		function s = getAxisSize(this)
			s = this.axisSize;
		end

		%%
		function c = getColor(this)
			c = this.color;
		end

		%%
		function setColor(this,c)
			this.color = c;
		end
	end

	methods (Abstract)
		%% Evaluates the surface at (u,v)
		x = evalSurf(this,u,v);

		%% Computes tangent points to xO and xI (both in object/surface space)
		[uO,vO,uI,vI,l] = computeTanPts(this,xO,xI,first);
	end

	methods (Static)
		%%
		function [x,iter,dx] = newton(evalFcn,xInit,tol,iterMax,dxMax)
			% Root finding: solve for g(x) = 0. The 'objective' function evalFcn
			% returns [f,g,H], but f is unused.
			if nargin < 5
				dxMax = 1e3;
			end
			if nargin < 4
				iterMax = 10*length(xInit);
			end
			if nargin < 3
				tol = 1e-9;
			end
			testGrad = false;
			x = xInit;
			iter = 1;
			while true
				[~,g,H] = evalFcn(x);
				if testGrad
					% Finite difference test
					sqrteps = sqrt(eps); %#ok<UNRCH>
					H_ = zeros(size(H));
					for i = 1 : length(x)
						x_ = x;
						x_(i) = x_(i) + sqrteps;
						[~,g_] = evalFcn(x_);
						H_(:,i) = (g_ - g)/sqrteps;
					end
					redmax.Scene.printError('H',H_,H);
				end
				% Newton direction
				dx = -H\g;
				if norm(dx) > dxMax
					fprintf('Newton diverged\n');
					break;
				end
				x = x + dx;
				% Convergence check
				if norm(g) < tol
					% Converged
					break;
				end
				if iter >= iterMax
					fprintf('Newton did not converge after %d iterations\n',iterMax);
					break;
				end
				iter = iter + 1;
			end
			%fprintf('%d\n',iter);
		end

	end
end
