#define _USE_MATH_DEFINES
#include <cstdio>
#include <random>
#include <cmath>
#include <iostream>
#include "time.h"

#include "WrapSphere.hpp"
#include "WrapCylinder.hpp"
#include "WrapDoubleCylinder.hpp"

using namespace Eigen;
using namespace std;

const int SAMPLE_SIZE = 3e3;
const int SAMPLES_PER_LINE = 50;
const bool IS_TRAINING_DATA = false;

VectorXf generate_random_vector(int, float, float);
void generateCylinder();
void generateSphere();
void print_vector(const VectorXf&, FILE*, bool);
void print_matrix(const MatrixXf&, FILE*);
Matrix4f get_child_dE(float);
Matrix4f get_child_E(float);

int main()
{
    generateCylinder();  
    // generateSphere();
    return 0;
}

void generateCylinder() {
    FILE* FILE_PTR_XTRAIN;
    const char* XTRAIN_FILENAME = "x_train_offline.txt";
    FILE* FILE_PTR_YTRAIN;
    const char* YTRAIN_FILENAME = "y_train_offline.txt";

    if (IS_TRAINING_DATA) {
        const char* XTRAIN_FILENAME = "x_train_offline.txt";
        const char* YTRAIN_FILENAME = "y_train_offline.txt";
    }
    else {
        XTRAIN_FILENAME = "x_test_offline.txt";
        YTRAIN_FILENAME = "y_test_offline.txt";
    }

    auto err1 = fopen_s(
        &FILE_PTR_XTRAIN,
        XTRAIN_FILENAME, "w");

    auto err2 = fopen_s(
        &FILE_PTR_YTRAIN,
        YTRAIN_FILENAME, "w");

    srand((unsigned int)time(0));

    VectorXf qr = generate_random_vector(SAMPLE_SIZE, -M_PI, M_PI);
    Matrix4f Ep = Matrix4f::Identity();
    int count = 0;

    for (int k = 0; k < SAMPLE_SIZE; k++) {
        Status status = Status::inside_radius;
        Eigen::Vector3f O, Z;
        O.setZero();
        Z.setZero();
        Z(2) = 1.0;
        Eigen::Vector4f xO, xI, xOw, xIw;

        Eigen::VectorXf r;
        Eigen::Vector3f points;
        float length;
        float total_length;
        WrapCylinder wc;

        while (status == Status::inside_radius) {
            xO = generate_random_vector(4, -1.0, 1.0);
            xI = generate_random_vector(4, -1.0, 1.0);
            xO(3) = 1.0f;
            xI(3) = 1.0f;

            xOw = xO;
            xIw = xI;

            r = generate_random_vector(1, 1e-2, 0.2);

            wc = WrapCylinder(xOw.segment<3>(0), xIw.segment<3>(0), O, Z, r(0));
            wc.compute();
            length = wc.getLength();
            total_length = wc.getTotalLength();
            status = wc.getStatus();
        }

        while (status == Status::wrap && length / total_length <=15e-2 && length > 1e-5) {

            xO = generate_random_vector(4, -1.0, 1.0);
            xI = generate_random_vector(4, -1.0, 1.0);
            xO(3) = 1.0f;
            xI(3) = 1.0f;

            xOw = xO;
            xIw = xI;
   
            r = generate_random_vector(1, 1e-2, 0.2);

            wc = WrapCylinder(xOw.segment<3>(0), xIw.segment<3>(0), O, Z, r(0));
            wc.compute();
            length = wc.getLength();
            total_length = wc.getTotalLength();
            status = wc.getStatus();
            count++;
        }

        for (int p = 0; p < SAMPLES_PER_LINE; p++) {
            float s = p / (SAMPLES_PER_LINE - 1.0);
            VectorXf input_offline(8); 
            input_offline << s, xOw.segment<3>(0), xIw.segment<3>(0), r(0);
            print_vector(input_offline, FILE_PTR_XTRAIN, false);

            VectorXf output_offline(3); 
            points = wc.getPoints(s);
            output_offline << points;
            if (!isnan(points(0)) && !isnan(points(1)) && !isnan(points(2))) {
                print_vector(output_offline, FILE_PTR_YTRAIN, false);
            }
            else {
                cout << "there is a nan output!" << endl;
            }
        }

        if (k % 500 == 0) {
            cout << (float)k / SAMPLE_SIZE * 100.0 << "% completed" << endl;
            cout << "throwed away: " << count << endl;
        }
    }

    cout << "completed generating data" << endl;

    fclose(FILE_PTR_XTRAIN);
    fclose(FILE_PTR_YTRAIN);
}

void generateSphere() {
    FILE* FILE_PTR_XTRAIN;
    const char* XTRAIN_FILENAME = "x_train_offline.txt";
    FILE* FILE_PTR_YTRAIN;
    const char* YTRAIN_FILENAME = "y_train_offline.txt";

    if (IS_TRAINING_DATA) {
        const char* XTRAIN_FILENAME = "x_train_offline.txt";
        const char* YTRAIN_FILENAME = "y_train_offline.txt";
    }
    else {
        XTRAIN_FILENAME = "x_test_offline.txt";
        YTRAIN_FILENAME = "y_test_offline.txt";
    }

    auto err1 = fopen_s(
        &FILE_PTR_XTRAIN,
        XTRAIN_FILENAME, "w");

    auto err2 = fopen_s(
        &FILE_PTR_YTRAIN,
        YTRAIN_FILENAME, "w");

    srand((unsigned int)time(0));

    Matrix4f Ep = Matrix4f::Identity();

    for (int k = 0; k < SAMPLE_SIZE; k++) {
        Status status = Status::inside_radius;
        Eigen::Vector3f O;
        O.setZero();
        Eigen::Vector4f xO, xI, xOw, xIw;
        Eigen::VectorXf r;
        Eigen::Vector3f point;
        float length;
        float total_length;
       
        WrapSphere ws;

        while (status == Status::inside_radius) {
            xO = generate_random_vector(4, -1.0, 1.0);
            xI = generate_random_vector(4, -1.0, 1.0);
            xO(3) = 1.0f;
            xI(3) = 1.0f;
            xOw = xO;
            xIw = xI;

            r = generate_random_vector(1, 5e-2, 0.2);

            ws = WrapSphere(xOw.segment<3>(0), xIw.segment<3>(0), O, 0.2);
            ws.compute();
            length = ws.getLength();
            total_length = ws.getTotalLength();
            status = ws.getStatus();
        }

        for (int p = 0; p < SAMPLES_PER_LINE; p++) {
            float s = p / (SAMPLES_PER_LINE - 1.0);
            VectorXf input_offline(7); 
            input_offline << s, xOw.segment<3>(0), xIw.segment<3>(0);
            print_vector(input_offline, FILE_PTR_XTRAIN, false);

            VectorXf output_offline(3);
            point = ws.getPoints(s);
            output_offline << point;
            print_vector(output_offline, FILE_PTR_YTRAIN, false);
        }

        if (k % 500 == 0) {
            cout << (float)k / SAMPLE_SIZE * 100.0 << "% completed" << endl;
        }
    }

    cout << "completed generating data" << endl;

    fclose(FILE_PTR_XTRAIN);
    fclose(FILE_PTR_YTRAIN);
}

void print_vector(const VectorXf& v, FILE* pFile, bool is_transpose = false) {
    if (is_transpose) {
        for (auto x : v) {
            fprintf_s(pFile, "%f\n", x);
        }
        return;
    }

    for (auto x : v) {
        fprintf_s(pFile, "%f ", x);
    }
    fprintf_s(pFile, "\n");
}

void print_matrix(const MatrixXf& m, FILE* pFile) {
    for (const auto &row : m.rowwise()) {
        print_vector(row, pFile);
    }
}

VectorXf generate_random_vector(int size, float min_val, float max_val) {
    VectorXf vals = VectorXf::Random(size);
    VectorXf base = VectorXf::Constant(size, -1.0);
    return ((max_val - min_val) / 2.0) * (vals - base) - min_val * base;
}

Matrix4f get_child_E(float qr) {
    Matrix4f Ec = Matrix4f::Identity();
    float s = sin(qr);
    float c = cos(qr);
    Ec(2, 2) = c;
    Ec(0, 0) = c;
    Ec(2, 0) = -s;
    Ec(0, 2) = s;
    return Ec;
}

Matrix4f get_child_dE(float qr) {
    Matrix4f dEc = Matrix4f::Zero();
    float s = sin(qr);
    float c = cos(qr);
    dEc(2, 2) = -s;
    dEc(0, 0) = -s;
    dEc(2, 0) = -c;
    dEc(0, 2) = c;
    return dEc;
}