clear;
rng(0);

NN = load('NNcylinderZ_thres_0.00_6_256.mat');

% n = 8; % # input
% m = 3; % # neurons in output layer
% m1 = 256; % # neurons in the hidden layers

% Evaluate network.
% Each column of input is: [a(k);xOw(1:3);xIw(1:3);radius];
nx = 20;

a = 0:1/(nx-1):1;
xIw = [-0.3,0.2,0.5];
xOw = [-0.2,-0.2,-0.7];
radius = 0.05; % 0.01~0.2
x = [0, xOw, xIw,radius]';
x = repmat(x,1,nx);
x(1,:) = a;
tic
[z,g] = evaluateReverse(x,NN);
toc
 
% Check gradient with finite differencing.
e = 1e-3;
g_ = zeros(18,nx);
for i = 1:6
	x_ = x;
	x_(i+1,:) = x_(i+1,:) + e;
	z_ = evaluateReverse(x_,NN);
	g_(3*i-2:3*i,:) = (z_ - z)/e;
end
fprintf('g check: %e\n',norm(g_ - g)/norm(g_));

%%
function [z,g] = evaluateReverse(x,NN)
% g is the gradient wrt the ith input
W1 = NN.W1;
b1 = NN.b1;
W2 = NN.W2;
b2 = NN.b2;
W3 = NN.W3;
b3 = NN.b3;
W4 = NN.W4;
b4 = NN.b4;
W5 = NN.W5;
b5 = NN.b5;
W6 = NN.W6;
b6 = NN.b6;
W7 = NN.W7;
b7 = NN.b7;
W8 = NN.W8;
b8 = NN.b8;

y1 = W1*x + b1;
z1 = tanh(y1);
y2 = W2*z1 + b2;
z2 = tanh(y2);
y3 = W3*z2 + b3;
z3 = tanh(y3);
y4 = W4*z3 + b4;
z4 = tanh(y4);
y5 = W5*z4 + b5;
z5 = tanh(y5);
y6 = W6*z5 + b6;
z6 = tanh(y6);
y7 = W7*z6 + b7;
z7 = tanh(y7);
y8 = W8*z7 + b8;
z = y8;

if nargout == 1
	return;
end

% Compute gradient in reverse mode

nx = size(x,2);
D1 = (1 - z1.^2)';
D2 = (1 - z2.^2)';
D3 = (1 - z3.^2)';
D4 = (1 - z4.^2)';
D5 = (1 - z5.^2)';
D6 = (1 - z6.^2)';
D7 = (1 - z7.^2)';

ix = 1 : nx;
iy = nx+1 : 2*nx;
iz = 2*nx+1 : 3*nx;
tmp7(ix,:) = D7 .* W8(1,:);
tmp7(iy,:) = D7 .* W8(2,:);
tmp7(iz,:) = D7 .* W8(3,:);

tmp6 = tmp7 * W7;
tmp6 = repmat(D6,3,1) .* tmp6;
tmp5 = tmp6 * W6;
tmp5 = repmat(D5,3,1) .* tmp5;
tmp4 = tmp5 * W5;
tmp4 = repmat(D4,3,1) .* tmp4;
tmp3 = tmp4 * W4;
tmp3 = repmat(D3,3,1) .* tmp3;
tmp2 = tmp3 * W3;
tmp2 = repmat(D2,3,1) .* tmp2;
tmp1 = tmp2 * W2;
tmp1 = repmat(D1,3,1) .* tmp1;

g = zeros(18,nx);
for i = 1:6
	W1i = W1(:,1+i);
	g(3*i-2,:) = tmp1(ix,:) * W1i;
	g(3*i-1,:) = tmp1(iy,:) * W1i;
	g(3*i,:) = tmp1(iz,:) * W1i;
end
end