function scene = scenes(sceneID,integrator)
% Creates test scenes

EULER = 1;
BDF1 = 2;
SDIRK2 = 3;

scene = redmax.Scene();
scene.integrator = integrator;

switch sceneID
	case 0
		scene.name = 'No Muscles';
		scene.h = 1e-2;
		scene.tEnd = 1.0;
		scene.grav = [0 0 -9.81]';
		scene.drawHz = 10;
		scene.plotH = false;
		scene.waxis = [-1 1 -0.1 0.1 -1 1];
		scene.Vexpected(EULER) = -6.0335418493213240e+00;
		scene.Vexpected(BDF1) = -7.8654383641369350e+00;
		scene.Vexpected(SDIRK2) = -5.7063874247215551e+00;

		% Bones
		dxy = 0.2;
		dz = 0.5;
		nbones = 3;
		sides = [dxy dxy dz];
		density = 1e3;

		for i = 1 : nbones
			if i == 1
				scene.bodies{i} = redmax.BodyCuboid(density,sides);
				scene.joints{i} = redmax.JointFixed([],scene.bodies{i});
				scene.joints{i}.setJointTransform([eye(3),[0,0,dz]'; 0 0 0 1]);
				scene.bodies{i}.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
			else
				scene.bodies{i} = redmax.BodyCuboid(density,sides);
				scene.joints{i} = redmax.JointRevolute(scene.joints{i-1},scene.bodies{i},[0 1 0]');
				scene.joints{i}.setJointTransform([eye(3),[0,0,-dz]'; 0 0 0 1]);
				scene.bodies{i}.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
				scene.joints{i}.setGeometry(dxy/4,dxy);
				scene.joints{i}.q(1) = 20*pi/180;
			end
			scene.joints{i}.setAxisSize(0.1);
			scene.bodies{i}.setAxisSize(0.1);
		end

		scene.initJoints();
		scene.initMuscles();

	case 1
		scene.name = 'Stationary Parent';
		scene.h = 1e-2;
		scene.tEnd = 1.0;
		scene.grav = [0 0 -9.81]';
		scene.drawHz = 30;
		scene.plotH = true;
		scene.waxis = [-1 1 -0.1 0.1 -1 1];
		scene.Vexpected(EULER) = -8.1085415203761868e-01;
		scene.Vexpected(BDF1) = -8.6100912538462471e-01;
		scene.Vexpected(SDIRK2) = -8.3481538776991671e-01;

		% Bones
		dxy = 0.2;
		dz = 0.5;
		sides = [dxy dxy dz];
		density = 0e3;
		bodyA = redmax.BodyCuboid(density,sides);
		jointA = redmax.JointFixed([],bodyA);
		jointA.setJointTransform([eye(3),[0,0,dz]'; 0 0 0 1]);
		jointA.setAxisSize(0.1);
		bodyA.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyA.setAxisSize(0.1);
		bodyB = redmax.BodyCuboid(density,sides);
		jointB = redmax.JointRevolute(jointA,bodyB,[0 1 0]');
		jointB.setJointTransform([eye(3),[0,0,-dz]'; 0 0 0 1]);
		jointB.setAxisSize(0.1);
		bodyB.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyB.setAxisSize(0.1);
		jointB.setGeometry(dxy/4,dxy);
		jointB.q(1) = 45*pi/180;
		scene.joints{1} = jointA;
		scene.joints{2} = jointB;
		scene.bodies{1} = bodyA;
		scene.bodies{2} = bodyB;

		% Populate transforms before adding muscles because the local positions
		% cannot be computed otherwise.
		scene.initJoints();

		% Discrete mass points for muscles
		massMuscle = 1;
		massPointCount = 10;
		as = linspace(0.0,1.0,massPointCount);
		ms = (massMuscle/massPointCount)*ones(1,massPointCount);
		muscle = redmax.MuscleI(as,ms,bodyA,[-dxy/2, dxy/2, 0.2]',bodyB,[-dxy/2, -dxy/2, -0.2]');
		muscle.name = 'muscle';
		scene.muscles{1} = muscle;

		% Initialize muscles
		scene.initMuscles();

	case 2
		scene.name = 'Moving Parent';
		scene.h = 1e-2;
		scene.tEnd = 1.0;
		scene.grav = [0 0 -9.81]';
		scene.drawHz = 30;
		scene.plotH = true;
		scene.waxis = [-1 1 -0.1 0.1 -1 1];
		scene.Vexpected(EULER) = -3.4657641433781770e+00;
		scene.Vexpected(BDF1) = -7.1588642753899805e+00;
		scene.Vexpected(SDIRK2) = -3.4926402394767919e+00;

		% Bones
		dxy = 0.2;
		dz = 0.5;
		sides = [dxy dxy dz];
		density = 0e0;
		bodyA = redmax.BodyCuboid(density,sides);
		jointA = redmax.JointFixed([],bodyA);
		jointA.setJointTransform([eye(3),[0,0,dz]'; 0 0 0 1]);
		jointA.setAxisSize(0.1);
		bodyA.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyA.setAxisSize(0.1);
		bodyB = redmax.BodyCuboid(density,sides);
		jointB = redmax.JointRevolute(jointA,bodyB,[0 1 0]');
		jointB.setJointTransform([eye(3),[0,0,-dz]'; 0 0 0 1]);
		jointB.setAxisSize(0.1);
		bodyB.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyB.setAxisSize(0.1);
		jointB.setGeometry(dxy/4,dxy);
		jointB.q(1) = 1;
		bodyC = redmax.BodyCuboid(density,sides);
		jointC = redmax.JointRevolute(jointB,bodyC,[0 1 0]');
		jointC.setJointTransform([eye(3),[0,0,-dz]'; 0 0 0 1]);
		jointC.setAxisSize(0.1);
		bodyC.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyC.setAxisSize(0.1);
		jointC.setGeometry(dxy/4,dxy);
		jointC.q(1) = 0;
		scene.joints{1} = jointA;
		scene.joints{2} = jointB;
		scene.joints{3} = jointC;
		scene.bodies{1} = bodyA;
		scene.bodies{2} = bodyB;
		scene.bodies{3} = bodyC;

		% Populate transforms before adding muscles because the local positions
		% cannot be computed otherwise.
		scene.initJoints();

		% Discrete mass points for muscles
		massMuscle = 10;
		massPointCount = 20;
		as = linspace(0,1,massPointCount);
		ms = (massMuscle/massPointCount)*ones(1,massPointCount);
		muscle = redmax.MuscleI(as,ms,bodyB,[-dxy/2, dxy/2, 0.2]',bodyC,[-dxy/2, -dxy/2, -0.2]');
		muscle.name = 'muscle';
		scene.muscles{1} = muscle;

		% Initialize muscles
		scene.initMuscles();

	case 3
		scene.name = 'Bi-articular';
		scene.h = 1e-2;
		scene.tEnd = 1.0;
		scene.grav = [0 0 -9.81]';
		scene.drawHz = 30;
		scene.plotH = true;
		scene.waxis = [-1 1 -0.1 0.1 -1 1];
		scene.Vexpected(EULER) = -3.9730928005426733e+00;
		scene.Vexpected(BDF1) = -1.9436694949871679e+01;
		scene.Vexpected(SDIRK2) = -6.6724437359684146e+00;

		% Bones
		dxy = 0.2;
		dz = 0.5;
		sides = [dxy dxy dz];
		density = 0e0;
		bodyA = redmax.BodyCuboid(density,sides);
		jointA = redmax.JointFixed([],bodyA);
		jointA.setJointTransform([eye(3),[0,0,dz]'; 0 0 0 1]);
		jointA.setAxisSize(0.1);
		bodyA.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyA.setAxisSize(0.1);
		bodyB = redmax.BodyCuboid(density,sides);
		jointB = redmax.JointRevolute(jointA,bodyB,[0 1 0]');
		jointB.setJointTransform([eye(3),[0,0,-dz]'; 0 0 0 1]);
		jointB.setAxisSize(0.1);
		bodyB.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyB.setAxisSize(0.1);
		jointB.setGeometry(dxy/4,dxy);
		jointB.q(1) = 1;
		bodyC = redmax.BodyCuboid(density,sides);
		jointC = redmax.JointRevolute(jointB,bodyC,[0 1 0]');
		jointC.setJointTransform([eye(3),[0,0,-dz]'; 0 0 0 1]);
		jointC.setAxisSize(0.1);
		bodyC.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyC.setAxisSize(0.1);
		jointC.setGeometry(dxy/4,dxy);
		jointC.q(1) = 0;
		bodyD = redmax.BodyCuboid(density,sides);
		jointD = redmax.JointRevolute(jointC,bodyD,[0 1 0]');
		jointD.setJointTransform([eye(3),[0,0,-dz]'; 0 0 0 1]);
		jointD.setAxisSize(0.1);
		bodyD.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyD.setAxisSize(0.1);
		jointD.setGeometry(dxy/4,dxy);
		jointD.q(1) = 0;
		scene.joints{1} = jointA;
		scene.joints{2} = jointB;
		scene.joints{3} = jointC;
		scene.joints{4} = jointD;
		scene.bodies{1} = bodyA;
		scene.bodies{2} = bodyB;
		scene.bodies{3} = bodyC;
		scene.bodies{4} = bodyD;

		% Populate transforms before adding muscles because the local positions
		% cannot be computed otherwise.
		scene.initJoints();

		% Uni-articular muscle
		massMuscle = 20;
		massPointCount = 20;
		as = linspace(0,1,massPointCount);
		ms = (massMuscle/massPointCount)*ones(1,massPointCount);
		muscle1 = redmax.MuscleI(as,ms,bodyB,[dxy/2, -dxy/2, 0.0]',bodyC,[dxy/2, dxy/2, -0.0]');
		muscle1.name = 'muscle1';
		scene.muscles{1} = muscle1;

		% Bi-articular muscle
		massMuscle = 20;
		massPointCount = 20;
		as = linspace(0,1,massPointCount);
		ms = (massMuscle/massPointCount)*ones(1,massPointCount);
		muscle2 = redmax.MuscleI(as,ms,bodyB,[-dxy/2, -dxy/2, 0.0]',bodyD,[-dxy/2, dxy/2, -0.0]');
		muscle2.name = 'muscle2';
		scene.muscles{2} = muscle2;

		% Initialize muscles
		scene.initMuscles();

	case 4
		scene.name = 'Path Points';
		scene.h = 1e-2;
		scene.tEnd = 1.0; % 1.0
		scene.grav = [0 0 -9.81]';
		scene.drawHz = 30;
		scene.plotH = true;
		scene.waxis = [-0.7 0.7 -0.1 0.1 -1 0];
		scene.wgrid = false;
		scene.Vexpected(EULER) = -1.3200002086856930e+00;
		scene.Vexpected(BDF1) = -1.4706635917226070e+00;
		scene.Vexpected(SDIRK2) = -1.3986497212444728e+00;

		% Bones
		dxy = 0.05;
		dz = 0.5;
		sides = [dxy dxy dz];
		density = 1e3;
		bodyA = redmax.BodyCuboid(density,1e-9*sides);
		jointA = redmax.JointFixed([],bodyA);
		jointA.setJointTransform([eye(3),[0,0,dz]'; 0 0 0 1]);
		bodyA.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyB = redmax.BodyCuboid(density,sides);
		jointB = redmax.JointRevolute(jointA,bodyB,[0 1 0]');
		jointB.setJointTransform([eye(3),[0,0,-dz]'; 0 0 0 1]);
		bodyB.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		jointB.setGeometry(dxy/4,dxy);
		jointB.q(1) = 0;
		bodyC = redmax.BodyCuboid(density,sides);
		jointC = redmax.JointRevolute(jointB,bodyC,[0 1 0]');
		jointC.setJointTransform([eye(3),[0,0,-dz]'; 0 0 0 1]);
		bodyC.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		jointC.setGeometry(dxy/4,dxy);
		jointC.q(1) = 1;
		scene.joints{1} = jointA;
		scene.joints{2} = jointB;
		scene.joints{3} = jointC;
		scene.bodies{1} = bodyA;
		scene.bodies{2} = bodyB;
		scene.bodies{3} = bodyC;
		for i = 1 : length(scene.bodies)
			scene.bodies{i}.setAxisSize(0);
			scene.joints{i}.setAxisSize(0);
		end

		% Populate transforms before adding muscles because the local positions
		% cannot be computed otherwise.
		scene.initJoints();

		% Discrete mass points for muscles
		massMuscle = density*prod(sides);
		massPointCount = 20;
		as = linspace(0,1,massPointCount);
		ms = (massMuscle/massPointCount)*ones(1,massPointCount);
		pathPoints = [];
		pathPoints(end+1).body = bodyB;
		pathPoints(end).xbody = [-dxy/2, 0, dz/2]';
		pathPoints(end+1).body = bodyB;
		pathPoints(end).xbody = [-0.1, 0, -dz/2+0.1]';
		pathPoints(end+1).body = bodyC;
		pathPoints(end).xbody = [-0.1, 0, dz/2-0.11]';
		pathPoints(end+1).body = bodyC;
		pathPoints(end).xbody = [-dxy/2, 0, -dz/2]';
		muscle = redmax.MuscleII(as,ms,pathPoints);
		muscle.name = 'muscle';
		scene.muscles{1} = muscle;

		% Initialize muscles
		scene.initMuscles();

	case 5
		scene.name = 'Wrap Cylinder Simple';

		% Note that the MLP evaluations in pure matlab is the bottleneck!

		scene.h = 2e-2;
		scene.tEnd = 1.5;
		scene.grav = [0 0 -9.81]';
		scene.drawHz = 30;

		scene.plotH = true;
		scene.waxis = 0.6*[-1 1 -0.1 0.1 -1 1];
		scene.wview = [0 0];
		scene.wgrid = true;

		scene.Vexpected(EULER) = -1.6092576980590820e+00;
		scene.Vexpected(BDF1) = -2.3955078125000000e+00;
		scene.Vexpected(SDIRK2) = -1.4844880104064941e+00;

		% Bones
		dxy = 0.05;
		dz = 0.5;
		sides = [dxy dxy dz];
		vol = prod(sides);
		density = 1e3;
		bodyA = redmax.BodyCuboid(density,sides);
		if true
			jointA = redmax.JointFixed([],bodyA);
		else
			jointA = redmax.JointRevolute([],bodyA,[0 1 0]); %#ok<UNRCH> 
			jointA.setGeometry(dxy/4,dxy);
		end
		jointA.setJointTransform([eye(3),[0,0,dz]'; 0 0 0 1]);
		jointA.setAxisSize(0.0);
		bodyA.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyA.setAxisSize(0.0);
		jointA.q(1) = 0;
		bodyB = redmax.BodyCuboid(density,sides);
		jointB = redmax.JointRevolute(jointA,bodyB,[0 1 0]);
		jointB.setJointTransform([eye(3),[0,0,-dz]'; 0 0 0 1]);
		jointB.setAxisSize(0.0);
		bodyB.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyB.setAxisSize(0.0);
		jointB.setGeometry(dxy/4,dxy);
		jointB.q(1) = -pi/2;
		scene.joints{1} = jointA;
		scene.bodies{1} = bodyA;
		scene.joints{2} = jointB;
		scene.bodies{2} = bodyB;

		% Populate transforms before adding muscles because the local positions
		% cannot be computed otherwise.
		scene.initJoints();

		% Muscles?
		if true
			% Discrete mass points for muscles
			massMuscle = 1.1*vol*density;
			massPointCount = 20;
			as = linspace(0,1,massPointCount);
			ms = (massMuscle/massPointCount)*ones(1,massPointCount);

			radius = 0.1;
			xA = [-dxy/2, 0, dz/2]';
			xB = [-dxy/2, 0, 0.125]';
			if false
				% Debug muscle
				E = [se3.aaToMat([1 0 0],pi/2), [0 0 -dz/2]'; 0 0 0 1]; %#ok<UNRCH>
				wrap = redmax.WrapCylinder(bodyA,E,radius,dxy,'-x');
				wrap.setAxisSize(0.0);
				muscle = redmax.MuscleIII_debug3(as,ms,bodyA,xA,bodyB,xB,wrap);
			else
				% NN muscle
				E = [se3.aaToMat([1 0 0],-pi/2), [0 0 -dz/2]'; 0 0 0 1];
				wrap = redmax.WrapCylinder(bodyA,E,radius,dxy,'-x');
				wrap.setAxisSize(0.0);
				muscle = redmax.MuscleIII(as,ms,bodyA,xA,bodyB,xB,wrap,'NN/NNcylinderZ.mat');
			end

			muscle.name = 'muscle';
			scene.muscles{1} = muscle;
		end

		% Initialize muscles
		scene.initMuscles();

	case 6
		scene.name = 'Wrap Cylinder Detach';

		% Note that the MLP evaluations in pure matlab is the bottleneck!

		% Set to true for debugging detaching muscles
		detach = true;

		scene.h = 2e-2;
		scene.tEnd = 1.0;
		scene.grav = [0 0 -9.81]';
		scene.drawHz = 30;
		scene.plotH = true;
		scene.waxis = [-1 1 -0.1 0.1 -1.2 0.2];
		scene.wview = [0 0];
		scene.wgrid = false;
		scene.Vexpected(EULER) = -9.4318695068359375e+00;
		scene.Vexpected(BDF1) = -1.2646072387695312e+01;
		scene.Vexpected(SDIRK2) = -1.0903167724609375e+01;

		% Bones
		dxy = 0.1;
		dz = 0.5;
		sides = [dxy dxy dz];
		density = 3e3; % mass = 15
		bodyA = redmax.BodyCuboid(density,1e-9*sides);
		jointA = redmax.JointFixed([],bodyA);
		jointA.setJointTransform([eye(3),[0,0,dz]'; 0 0 0 1]);
		jointA.setAxisSize(0.0);
		bodyA.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyA.setAxisSize(0.0);
		bodyB = redmax.BodyCuboid(density,sides);
		jointB = redmax.JointRevolute(jointA,bodyB,[0 1 0]');
		jointB.setJointTransform([eye(3),[0,0,-dz]'; 0 0 0 1]);
		jointB.setAxisSize(0.0);
		jointB.setGeometry(dxy/8,dxy);
		bodyB.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyB.setAxisSize(0.0);
		bodyC = redmax.BodyCuboid(density,sides);
		jointC = redmax.JointRevolute(jointB,bodyC,[0 1 0]');
		jointC.setJointTransform([eye(3),[0,0,-dz]'; 0 0 0 1]);
		jointC.setAxisSize(0.0);
		bodyC.setBodyTransform([eye(3),[0,0,-dz/2]'; 0 0 0 1]);
		bodyC.setAxisSize(0.0);
		jointC.setGeometry(dxy/8,dxy);
		if detach
			jointB.q(1) = 0;
			jointC.q(1) = -1;
		else
			jointB.q(1) = 1; %#ok<UNRCH> 
			jointC.q(1) = 0;
		end
		scene.joints{1} = jointA;
		scene.joints{2} = jointB;
		scene.joints{3} = jointC;
		scene.bodies{1} = bodyA;
		scene.bodies{2} = bodyB;
		scene.bodies{3} = bodyC;

		% Populate transforms before adding muscles because the local positions
		% cannot be computed otherwise.
		scene.initJoints();

		% Discrete mass points for muscles
		massMuscle = 10;
		massPointCount = 20;
		as = linspace(0,1,massPointCount);
		ms = (massMuscle/massPointCount)*ones(1,massPointCount);

		radius = 0.15;
		xB = [-dxy/2, 0, 0.2]';
		xC = [-dxy/2, 0, -0.2]';

		E = [se3.aaToMat([1 0 0],-pi/2), [0 0 dz/2]'; 0 0 0 1];
		wrap = redmax.WrapCylinder(bodyC,E,radius,0.3,'-x');
		wrap.setAxisSize(0);
		muscle = redmax.MuscleIII(as,ms,bodyB,xB,bodyC,xC,wrap,'NN/NNcylinderZ.mat');

		muscle.name = 'muscle';
		scene.muscles{1} = muscle;

		% Initialize muscles
		scene.initMuscles();

	otherwise
		error('Invalid scene ID');
end

end
